#include <stdio.h>
#include <assert.h>
#include <string.h>
#include <stdlib.h>
#include "defines.h"
#include "foto.h"
#include "readField.h"

#define SEP ';'
#define FORMATO_STATO "%c;"
#define EOL 10



BOOLEAN leggiFotoSingola(FILE *f, FotoType *foto);

ListaFotoType leggiFoto(char *fileName)
{
    ListaFotoType lista = NULL;
    FotoType foto;
    FILE *f = fopen(fileName, "r");
    assert(f != NULL);
    while (leggiFotoSingola(f, &foto))
    {
        lista = aggiungiFoto(lista, &foto);
    }
    fclose(f);
    return lista;
}

BOOLEAN leggiFotoSingola(FILE *f, FotoType *foto)
{
    char buffer[FOTO_MAXCHARS];

    if (readField(foto->nome, SEP, f) != SEP)
        return FALSE;
    if (readField(foto->fotografo, SEP, f) != SEP)
        return FALSE;
    if (fscanf(f, FORMATO_STATO, &foto->stato) == 0)    //Almeno una parola chiave!
        return FALSE;
    foto->keyWords.count = 0;
    while (readField(buffer, SEP, f) == SEP)    //Almeno una parola chiave!
    {
        if (!aggiungiKeyWord(&(foto->keyWords), buffer))
            return FALSE;
    }
    return aggiungiKeyWord(&(foto->keyWords), buffer); //Aggiunge l'ultima keyword (il separatore non  pi SEP)
}

ListaFotoType aggiungiFoto(ListaFotoType lista, FotoType *foto)
{
    ListaFotoType nuovaLista = (ListaFotoType)malloc(sizeof(struct listaFotoStruct));
    nuovaLista->foto = *foto;
    nuovaLista->next = lista;
    return nuovaLista;
}

BOOLEAN aggiungiKeyWord(KeyWordsType *keyWords, char keyWord[])
{
    if (keyWords->count < FOTO_MAXKEYWORDS)
    {
        strcpy(keyWords->words[keyWords->count++], keyWord);
        return TRUE;
    }
    else
    {
        return FALSE;
    }
}

BOOLEAN trovaKeyWord(KeyWordsType *keyWords, char keyWord[])
{
    int i;
    for (i = 0; i < keyWords->count; i++)
    {
        if (strcmp(keyWords->words[i], keyWord) == 0)
            return TRUE;
    }
    return FALSE;
}

void stampaFotoSingola(const FotoType *foto)
{
    int i;
    printf("%s -- %s -- %s\n", foto->nome, foto->fotografo, foto->stato == 'F' ? "Pubblica" : "Privata");
    printf("KeyWords: ");
    for (i = 0; i < foto->keyWords.count; i++)
    {
        printf("%s; ", foto->keyWords.words[i]);
    }
    printf("\n");
}

int compareFoto(FotoType *foto1, FotoType *foto2)
{
    return strcmp(foto1->nome, foto2->nome);
}

ListaFotoType insOrdFoto(ListaFotoType lista, FotoType *foto)
{
    if (lista != NULL && compareFoto(foto, &(lista->foto)) > 0)
    {
        lista->next = insOrdFoto(lista->next, foto);
        return lista;
    }
    else
    {
        return aggiungiFoto(lista, foto);
    }
}

ListaFotoType ordinaFoto(ListaFotoType lista, char *username)
{
    ListaFotoType listaOrdinata = NULL;
    while (lista != NULL)
    {
        if (strcmp(lista->foto.fotografo, username) == 0)
            listaOrdinata = insOrdFoto(listaOrdinata, &(lista->foto));
        lista = lista->next;
    }
    return listaOrdinata;
}

ListaFotoType ricercaFoto(ListaFotoType lista, char *username, char *keyWord)
{
    ListaFotoType listaRicerca = NULL;
    while (lista != NULL)
    {
        if ((lista->foto.stato == 'F' || strcmp(lista->foto.fotografo, username) == 0) && 
            trovaKeyWord(&(lista->foto.keyWords), keyWord))
        {
            listaRicerca = aggiungiFoto(listaRicerca, &(lista->foto));
        }
        lista = lista->next;
    }
    return listaRicerca;
}

void stampaFoto(ListaFotoType lista)
{
    while (lista != NULL)
    {
        stampaFotoSingola(&(lista->foto));
        lista = lista->next;
    }
}

void destroyListaFoto(ListaFotoType lista)
{
    if (lista != NULL)
    {
        destroyListaFoto(lista->next);
        free(lista);
    }
}

